/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.utils.RegistryUtils;
import org.wso2.registry.jdbc.handlers.filters.Filter;

import java.util.*;

public class HandlerManager {

    private Map <Filter, Handler> getHandlerMap = new HashMap <Filter, Handler> ();
    private Map <Filter, Handler> putHandlerMap = new HashMap <Filter, Handler> ();
    private Map <Filter, Handler> deleteHandlerMap = new HashMap <Filter, Handler> ();
    private Map <Filter, Handler> importHandlerMap = new HashMap <Filter, Handler> ();
    private Map <Filter, Handler> putChildHandlerMap = new HashMap <Filter, Handler> ();
    private Map <Filter, Handler> importChildHandlerMap = new HashMap <Filter, Handler> ();

    public void addHandler(String[] methods, Filter filter, Handler handler) {

        if (methods == null || RegistryUtils.containsString(Filter.GET, methods)) {
            getHandlerMap.put(filter, handler);
        }

        if (methods == null || RegistryUtils.containsString(Filter.PUT, methods)) {
            putHandlerMap.put(filter, handler);
        }

        if (methods == null || RegistryUtils.containsString(Filter.DELETE, methods)) {
            deleteHandlerMap.put(filter, handler);
        }

        if (methods == null || RegistryUtils.containsString(Filter.IMPORT, methods)) {
            importHandlerMap.put(filter, handler);
        }

        if (methods == null || RegistryUtils.containsString(Filter.PUT_CHILD, methods)) {
            putChildHandlerMap.put(filter, handler);
        }

        if (methods == null || RegistryUtils.containsString(Filter.IMPORT_CHILD, methods)) {
            importChildHandlerMap.put(filter, handler);
        }
    }

    public Resource get(RequestContext requestContext) throws RegistryException {

        Resource resource = null;

        Iterator<Filter> filters = getHandlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleGet(requestContext)) {
                Handler handler = getHandlerMap.get(filter);
                resource = handler.get(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }

        return resource;
    }

    public String put(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = putHandlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handlePut(requestContext)) {
                putHandlerMap.get(filter).put(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }

        if (requestContext.isProcessingComplete()) {
            return requestContext.getActualPath();
        } else {
            return null;
        }
    }

    public String importResource(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = importHandlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleImportResource(requestContext)) {
                importHandlerMap.get(filter).importResource(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }

        if (requestContext.isProcessingComplete()) {
            return requestContext.getActualPath();
        } else {
            return null;
        }
    }

    public void delete(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = deleteHandlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleDelete(requestContext)) {
                deleteHandlerMap.get(filter).delete(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }
    }

    public void putChild(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = putChildHandlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handlePutChild(requestContext)) {
                putChildHandlerMap.get(filter).putChild(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }
    }

    public void importChild(RequestContext requestContext) throws RegistryException {

        Iterator<Filter> filters = importChildHandlerMap.keySet().iterator();
        while (filters.hasNext()) {
            Filter filter = filters.next();
            if (filter.handleImportChild(requestContext)) {
                importChildHandlerMap.get(filter).importChild(requestContext);
            }

            if (requestContext.isProcessingComplete()) {
                break;
            }
        }
    }
}
